#version 330
#extension GL_ARB_separate_shader_objects : enable





layout(location = 0) smooth in vec4 _fin_fragColor0;
layout(location = 1) smooth in vec4 _fin_fragColor1;
layout(location = 2) smooth in vec4 _fin_fragColor2;
layout(location = 3) smooth in vec4 _fin_fragNodeExtendedPixelCoord;
layout(location = 4) smooth in vec4 _fin_fragNodePixelCoord;

struct SFragmentInput
{
	vec4 fragColor0;
	vec4 fragColor1;
	vec4 fragColor2;
	vec4 fragNodeExtendedPixelCoord;
	vec4 fragNodePixelCoord;
	bool IsFrontFace;
};

layout(location = 0) out vec4 _fout_outColor;

struct SFragmentOutput
{
	vec4 outColor;
};




void FragmentMain( in SFragmentInput  fInput,  out SFragmentOutput  fOutput  )
{

	if (fInput.fragNodePixelCoord.x < 0.0f || fInput.fragNodePixelCoord.x > fInput.fragNodePixelCoord.z)
	{
		float lineWidth = 1.0f;
		float invCoordX = fInput.fragNodeExtendedPixelCoord.z - fInput.fragNodeExtendedPixelCoord.x;
		bool isLine = abs(invCoordX - fInput.fragNodeExtendedPixelCoord.y) < lineWidth;
		fOutput.outColor = isLine ? vec4(fInput.fragColor0.xyz, 1.0f / 3.0f) : vec4(0.0f, 0.0f, 0.0f, 0.0f);
	}
	else
	{


		float dashSize = 5.0f;
		int dashFrequency = 3;
		float coord = fInput.fragNodePixelCoord.x + fInput.fragNodePixelCoord.y;
		vec4 colDash = (int((coord + 0) / dashSize) % dashFrequency == 0) ? fInput.fragColor1 : fInput.fragColor0;
		vec4 colDashPos = (int((coord + 1) / dashSize) % dashFrequency == 0) ? fInput.fragColor1 : fInput.fragColor0;
		vec4 colDashNeg = (int((coord - 1) / dashSize) % dashFrequency == 0) ? fInput.fragColor1 : fInput.fragColor0;
		fOutput.outColor = mix(colDash, mix(colDashPos, colDashNeg, 0.5f), 1.0f / 3.0f);

		vec2 borderSize = vec2(0.0f, 2.0f);
		if ( any( lessThan(fInput.fragNodePixelCoord.xy, borderSize) )  ||
			any( lessThan((fInput.fragNodePixelCoord.zw - fInput.fragNodePixelCoord.xy), borderSize) ) )
			fOutput.outColor = fInput.fragColor2;
	}
}


void main()
{
	SFragmentInput fInput;
	SFragmentOutput fOutput;

	fInput.fragColor0 = _fin_fragColor0;
	fInput.fragColor1 = _fin_fragColor1;
	fInput.fragColor2 = _fin_fragColor2;
	fInput.fragNodeExtendedPixelCoord = _fin_fragNodeExtendedPixelCoord;
	fInput.fragNodePixelCoord = _fin_fragNodePixelCoord;
	fInput.IsFrontFace = gl_FrontFacing;
	FragmentMain(fInput, fOutput);
	_fout_outColor = fOutput.outColor;
}
